;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Housing Market Model
;;Sarah Ustvedt
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

extensions [gis]

globals
[
  center              ; geographic center of Oslo
  mapData             ; GIS-data for locating houses in grid
  interestRates       ; exogenous interest rates for simulation period
  interestRate        ; exogenous interest rate for current period
  depositRates        ; exogenous deposit rates for simulation period
  depositRate         ; exogenous interest rate for current period
  initDebt            ; debt used to instantiate households
  movingProbability   ; age-dependent probability of moving
  markup              ; interest increase that must be affordable for households used for mortgage extension
  investmentMarkup    ; markup on deposit rate for housing investments
  nSearch             ; number of houses a household potentially bids on in a period
  unemploymentRates   ; exogenous unemployment rates for simulation period
  unemploymentRate    ; exogenous unemployment rate for current period
  periodSales         ; list of sold houses in the previous month
  quarterSales        ; list of sold houses in the previous quarter
  yearSales           ; list of houses sold in previous year
  yearAppreciation    ; appreciation of houses in the last year
  savingsRates        ; age-dependent savings rates
  construction        ; vector of constructed houses
  yearBuilds          ; accumulated yearly dwellings added to the simulation
  netImmigration      ; immigration rates
  housingExpenditure  ; income-dependent expenditure
  year                ; current year
  month               ; current month
  incomeDeciles       ; current income deciles of households
  minYield            ; minimum yield on housing for investors to buy
  monthInterest       ; monthly interest rate (converted from yearly)
  ltvLimit            ; limit on LTV of extended mortgages (1 for all years in benchmark case)
  ltvInvest           ; minimum LTV for investors (to calculate yield)
  consumptionBudget   ; minimum consumption budget for family of four (divided by two for households without children)
  interestOnly        ; proportion of loans allowed to be interest-only in current year
  freeExtended        ; number of extended mortgages with interest-only period in current year
  periodSqm           ; square meter price in period
]


breed [banks bank]
banks-own
[
  myMortgages
  assets
  liabilities
]



breed [households household]
households-own
[
  age
  householdCategory ; defines whether household is young or old, single or married, has child or not
  income            ; household monthly income
  incomePast        ; income before unemployment period
  incomePrev        ; Z_{h,t-1} used for stochastic income process
  myHome            ; dwelling household owns and lives in
  myRental          ; dwelling household rents and lives in
  myInvestments     ; houses bought as investments
  myMortgages       ; mortgages owned by household
  debt              ; total debt of all mortgages
  savings           ; total savings
  hasChild          ; 1 if household has child, 0 else
  ageChild          ; age of youngest child
  bidList           ; list of houses household may enter bidding round for in each time period
  currentBid        ; current bid by household for preferred house in bidlist
  isUnemployed      ; 1 if household is unemployed, 0 else
  initSale          ; price payed for house bought during initialization
  approvedMortgage  ; mortgage approved by bank that can be used to enter housing market
]


breed [houses house]
houses-own
[
  houseCategory  ; house, duplex, semi-detachable or apartment
  zone           ; district house is located in
  distCenter     ; distance to city center
  timeOnMarket   ; time house has been for sale
  onMarket       ; 1 if house is listed on the market, 0 otherwise
  listingPrice   ; price house is listed on the market for
  salesPrice     ; price last time house was sold
  bidders        ; list of households who have submitted bids for house
  isRental       ; 1 if house is rental, 0 otherwise
  yearRent       ; cost of renting house for a year
  myOwner        ; household owning house
  myRenter       ; household renting house
  soldAt         ; period the house was last sold at
  sqm            ; size of house
]



breed [mortgages mortgage]
mortgages-own
[
  principle       ; remaining mortgage to be payed
  maturity        ; remaining time for paying back loan
  mortgageOwner   ; household who owns mortgage
  mortgageHouse   ; house connected to the mortgage
  LTV             ; loan-to-value at origin
  timeStamp       ; period the loan was extended
  freePeriod      ; 1 if loan has interest-only period
  initDTI         ; DTI at issue
  downPayment     ; patyment to reduce principle in period
  ageHousehold    ; age of household owning mortgage
]


patches-own
[
  inZone;
]


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Initialization
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


to setup
  __clear-all-and-reset-ticks
  setup-map
  setup-params
  setup-houses
  setup-households
  updateDeciles
  setup-market
  setup-housing-price-plot
end


; sets up the map of the city using GIS dataset
to setup-map
  let zoneList [9 5 1 10 2 4 8 14 13 3 0 15 11 6 7 12]
  set mapData gis:load-dataset "gisData/oslo.shp"
  gis:set-drawing-color white
  gis:draw mapData 1
  let i 0
  foreach gis:feature-list-of mapData [
    if item i zoneList = 0 [
        set center patch (item 0 gis:location-of(gis:centroid-of ?)) (item 1 gis:location-of(gis:centroid-of ?))
        ask center [
          set pcolor 5
        ]
    ]
    ask patches gis:intersecting ? [
      set inZone item i zoneList
    ]
    set i i + 1
  ]
end



; initializing all parameters
to setup-params
 set initDebt [[0.485  0.739  0.826  0.87  0.893  0.907  1][  0.36  0.706  0.827  0.891  0.928  0.951  1][  0.271  0.655  0.787  0.868  0.921  0.953  1 ][ 0.202  0.576  0.728  0.845  0.922  0.961  1 ][ 0.155  0.516  0.695  0.84  0.928  0.965  1 ][ 0.114  0.465  0.682  0.85  0.937  0.97  1  ][ 0.07  0.402  0.664  0.868  0.951  0.977  1 ][  0.046  0.371  0.675  0.889  0.96  0.981  1 ][ 0.036  0.389  0.722  0.908  0.966  0.984  1 ][ 0.033  0.442  0.769  0.916  0.962  0.98  1]]
 set movingProbability [0.017406806 0.010596241 0.005143013 0.003396053 0.002535049 0.002535049 0.002535049]
 set markup 0.04
 set nSearch 10
 set unemploymentRates [0.039  0.036  0.036  0.035  0.033  0.034  0.036  0.035  0.032  0.03  0.028  0.027  0.028  0.027  0.026  0.025  0.024  0.024  0.027  0.026  0.024  0.022  0.021  0.022  0.022  0.021  0.021  0.02  0.019  0.02  0.023  0.023  0.022  0.022  0.023  0.025  0.029  0.03  0.032  0.033  0.032  0.035  0.039  0.038  0.036  0.035  0.034  0.035  0.039  0.037  0.037  0.038  0.036  0.037  0.041  0.04  0.039  0.037  0.036  0.035  0.038  0.036  0.034  0.034  0.031  0.031  0.035  0.033  0.031  0.031  0.029  0.03  0.032  0.031  0.031  0.031  0.029  0.03  0.034  0.033  0.032  0.031  0.03  0.03  0.033  0.033  0.033  0.033  0.032  0.032  0.036  0.036  0.034  0.034  0.034  0.034  0.037  0.037  0.037  0.035  0.035  0.035  0.038  0.037  0.035  0.034  0.033  0.033  0.031  0.031  0.031  0.031  0.031  0.031  0.031  0.031  0.031  0.031  0.031  0.031]
 set unemploymentRate item 0 unemploymentRates
 set depositRates [0.0164  0.018  0.02  0.0234  0.0284  0.0304  0.0355  0.041  0.0452  0.0474  0.0522  0.0473  0.0288  0.0191  0.0172  0.019  0.0201  0.0204  0.0216  0.022  0.0223  0.0221  0.0242  0.0258  0.0259  0.0233  0.0237  0.0246  0.0247  0.0229  0.0238  0.0242  0.0243  0.0219  0.022  0.0198  0.0163  0.0151  0.0123  0.0092]
 set depositRate item 0 depositRates
 set interestRates [0.0347  0.0366  0.0378  0.0406  0.0465  0.0496  0.0548  0.0604  0.0641  0.0676  0.0716  0.0703  0.0471  0.0383  0.0363  0.037  0.0384  0.0394  0.0401  0.0397  0.0393  0.0392  0.0413  0.0431  0.0429  0.0412  0.0409  0.0407  0.0405  0.0423  0.0422  0.0416 0.0415  0.0402  0.0396  0.0377  0.0341  0.0332  0.0308  0.0286]
 set interestRate item 0 interestRates
 set monthInterest (1 + interestRate)^(1 / 12) - 1
 set periodSales no-turtles
 set quarterSales no-turtles
 set savingsRates [0.07 0.10 0.07 0.09 0.13 0.12 0.14]
 set housingExpenditure [0.278 0.239 0.239 0.208 0.208 0.157 0.157 0.141 0.141 0.13]
 set yearBuilds [0 39 76 102 136 156 169 207 237 267 308]
 set construction []
 set netImmigration [97  106  104  88  99  110  98  91  100 100]
 set yearAppreciation 1.08
 set year 1
 set month 1
 set incomeDeciles []
 set investmentMarkup 0.05
 set minYield depositRate + investmentMarkup
 set ltvLimit [1 1 1 1 1 1 1 1 1 1]
 set consumptionBudget [227360  228930  232100  234454  247390  246910  250750  250150  256850  259570]
 set interestOnly [0.17  0.2  0.17  0.17  0.17  0.25  0.17  0.12  0.14  0.11]
 set freeExtended 0
 set periodSqm 30000
end





; loads dwellings from file and places them in appropriate geographical location
to setup-houses
  file-close
  file-open "init_houses.txt"
  let i 0
  while [not file-at-end?] [
    create-houses 1 [
      set houseCategory file-read
      set listingPrice file-read
      set salesPrice listingPrice
      set zone file-read
      set sqm file-read
      set timeOnMarket 0
      set onMarket 1
      set bidders []
      set myOwner nobody
      set timeOnMarket 0
      setColor listingPrice
      move-to one-of patches with [inZone = [zone] of myself]
      set distCenter distance center
    ]
  ]
  file-close
  file-open"update_houses.txt"
  while [not file-at-end?][
    let temp1 file-read
    let temp2 file-read
    let temp3 file-read
    let temp4 file-read
    let tempVec (list temp1 temp2 temp3 temp4)
    set construction lput tempVec construction
  ]
end

; load households from file
to setup-households
  file-close
  file-open "init_households.txt"
  while [not file-at-end?] [
    create-households 1 [
      set householdCategory file-read
      set isUnemployed 0
      set age file-read
      set income file-read
      set savings file-read
      set hasChild file-read
      set incomePrev income
      set myHome nobody
      set myRental nobody
      set bidList []
      set myInvestments []
      set myMortgages no-turtles
      if hasChild = 1 [
        set ageChild random 18
      ]
    ]
  ]
end

; assign houses to households initially using empirical debt values to determine debt
to setup-market
  ask households [
    let incDec incomeDecile income
    let debtVec item (incDec - 1) initDebt
    let rand random-float 1
    let notFinished true
    let index 0
    let myDebt 0
    while [notFinished] [
      if rand <= item index debtVec [
        set myDebt income * (index + random-float 1)
        set notFinished false
      ]
      set index index + 1
    ]
    if one-of houses with [myDebt + [savings] of myself > listingPrice  and onMarket = 1] != nobody [
       let potentialHomes houses with [myDebt + [savings] of myself > listingPrice and onMarket = 1]
       let sortedList sort-by [[listingPrice] of ?1 > [listingPrice] of ?2] potentialHomes
       set myHome item 0 sortedList
       ask myHome [
         set myOwner myself
         set salesPrice listingPrice
         set onMarket 0
       ]
    ]
  ]
  create-mortgages count households with [myHome != 0] [
    set timeStamp ticks
    set mortgageOwner nobody
  ]
  ask households with [myHome != nobody] [
    set initSale [salesPrice] of myHome
    ask one-of mortgages with [timeStamp = ticks and mortgageOwner = nobody] [
      set mortgageOwner myself
      set ageHousehold [age] of myself
      set principle [initSale] of myself - [savings] of myself
      set initDTI principle / [income] of myself
      set maturity 25 * 12
      set freePeriod 0
      set mortgageHouse nobody
      if principle < 0 [
        die
      ]
    ]
    ifelse savings > [salesPrice] of myHome [
      set savings savings - [listingPrice] of myHome
      ][
      set debt [listingPrice] of myHome - savings
      set savings 0
      ]
    set myMortgages mortgages with [mortgageOwner = myself]
  ]
end

; set color of houses on map based on price
to setColor [price]
  ifelse price < 2000000 [
        set color lime
      ][ ifelse price < 3000000 [
        set color green
      ][ ifelse price < 4000000 [
        set color orange
      ][
        set color red
      ]]]

end

; update income deciles of model
to updateDeciles
  let sortedIncomes sort-on [income] households
  let index round ((count households) / 10)
  let newDeciles []
  let i 1
  while [i <= 9] [
    set newDeciles lput ([income] of item (index * i) sortedIncomes) newDeciles
    set i i + 1
  ]
  set incomeDeciles newDeciles
end

; get income decile of household
to-report incomeDecile [inc]
  let myDecile 10
  let decile 0
  let notFound true
  while [notFound and decile < 9] [
    if (inc < item decile incomeDeciles)  [
      set notFound false
      set myDecile decile + 1
    ]
    set decile decile + 1
  ]
  report myDecile
end


; get inceom-dependent housing expenditure
to-report preferredExp [inc]
  let incomeDec incomeDecile inc
  report item (incomeDec - 1) housingExpenditure
end


; handle mortgage applications from households
to-report initialApplication [inc currDebt saveAge child residence]
  let preferredExpenditure preferredExp inc
  let dispInc inc * (1 - getSavings saveAge)
  let initialMortgage (dispInc * preferredExpenditure) / interestRate
  if initialMortgage + currDebt > inc * DTI [
    set initialMortgage inc * DTI - currDebt
  ]
  let mortgageCost (initialMortgage * (interestRate + markup)*(1 - 0.28)) / (1 - (1 + (interestRate + markup) * (1 - 0.28))^(-25))
  let minConsumption 0
  ifelse child = 1 [
    set minConsumption item (year - 1) consumptionBudget
  ][
    set minConsumption item (year - 1) consumptionBudget / 2
  ]
  if inc - mortgageCost - currDebt * interestRate  < minConsumption [
    let interestOnlyPeriod 0
    set interestOnlyPeriod interestOnlyCheck
     ifelse interestOnlyPeriod = 1 [
       set initialMortgage (inc - minConsumption - currDebt * interestRate) / ((interestRate + markup) * (1 - 0.28))

     ][
       set initialMortgage (inc - minConsumption - currDebt * interestRate) * ((1 - (1 + (interestRate + markup) * (1 - 0.28))^(-25)) / ((interestRate + markup)*(1 - 0.28)))
     ]
  ]
  report initialMortgage
end



; give proportion of households interest only period on mortgages
to-report interestOnlyCheck
  ifelse freeExtended < count mortgages with [timeStamp = ticks] * (item (year - 1) interestOnly) [
    set freeExtended freeExtended + 1
    report 1
  ][
    report 0
  ]
end


; create mortgages for all households who can afford house
to mortgageApplications
  ask mortgages with [mortgageHouse = nobody] [
    die
  ]
  let cheapestHouse min [listingPrice] of houses with [onMarket = 1]
  create-mortgages count households with [savings + DTI * income > cheapestHouse] [
    set timeStamp ticks
    set mortgageOwner nobody
  ]
  ask households with [savings + DTI * income > cheapestHouse and income > 0]  [
    ask one-of mortgages with [timeStamp = ticks and mortgageOwner = nobody] [
      set mortgageOwner myself
      set ageHousehold [age] of myself
      set principle initialApplication [income] of myself [debt] of myself [age] of myself [hasChild] of myself [myHome] of myself
      set initDTI principle / [income] of myself
      set maturity 25 * 12
      set mortgageHouse nobody
    ]
    set approvedMortgage one-of mortgages with [mortgageOwner = myself and timeStamp = ticks]
  ]

end


; main function run once in each time period
to go
   tick
   if ticks mod 12 = 0 and ticks > 0 [
     updateLifecycle      ; age and household category update
     updateIncome         ; income follows stochastic process
     updateUnemployment
     set year year + 1
     set month 0
     set freeExtended 0
     update-year-price-plot
     update-mean-DTI-plot
     total-debt-plot
     update-rentals-plot
   ]
   payDebt
   updateSavings
   newListings
   updateListings       ; new houses added to market and existing house prices reduces
   mortgageApplications
   clearMarket
   calcAppreciation
   updateRents
   rentHome
   buildHouses
   dieElderly
   updateImmigration
   updateParams
   update-housing-price-plot
   update-ownership-plot
   update-salesListing-plot


   set periodSales []

end


; update parameters according to period
to updateParams
  let quarter (ticks - ticks mod 3) / 3
  set interestRate item quarter interestRates
  set depositRate item quarter depositRates
  set unemploymentRate item (ticks - 1) unemploymentRates
  set month month + 1
  updateDeciles
  set minYield depositRate + investmentMarkup
  set monthInterest (1 + interestRate)^(1 / 12) - 1
  if count periodSales > 0 [
    set periodSqm mean [salesPrice / sqm] of periodSales
  ]
end


; update lifecyle of households
to updateLifecycle
  ask households
  [
    set age age + 1
    let probChild 0
    ifelse age <= 24 [
      set probChild 0.06
    ][
     ifelse age <= 29 [
       set probChild 0.13
     ][
      ifelse age <= 34 [
        set probChild 0.12
      ][
       ifelse age <= 39 [
         set probChild 0.05
       ][
         set probChild 0
       ]]]]
    let draw random 100
    if draw < probChild * 100 [
      set hasChild 1
      set ageChild -1
    ]
    if hasChild = 1 [
      set ageChild ageChild + 1
    ]
    if ageChild > 18 [
      set hasChild 0
    ]
  ]

end


; remove elderly households from model
to dieElderly
  let elderly households with [age >= 79]
  let monthDie round(count elderly / (13 - month))
  ask n-of monthDie elderly [
     if myHome != nobody [
      ask myHome [
        set onMarket 1
        set myOwner nobody
       ]]
     ask myMortgages [
       die
     ]
     if myRental != nobody [
      ask myRental [
        set myRenter nobody
      ]
    ]
    die
    ]
end


; match unemployment to historical figures
to updateUnemployment
  ask households with [isUnemployed = 1][
    set isUnemployed 0
    set income incomePast
  ]
  ask n-of round(unemploymentRate * count households) households [
    set incomePast income
    set income item 0 incomeDeciles
    set isUnemployed 1
  ]
end

; update income of households
to updateIncome
  ask households
  [
   let growth getGrowth age
   let transitoryIncome incomePrev + income * (growth + 0.02) + random-normal 0 0.032
   set incomePrev transitoryIncome
   set income transitoryIncome + random-normal 0 0.037
    ]
end


; get income growth rate of households
to-report getGrowth [ages]
  let growth 0
  ifelse ages < 29 [
    set growth 0.1
  ][
   ifelse ages < 59 [
     set growth 0.05
   ][
    ifelse ages < 69 [
      set growth -0.05
    ][
    set growth 0
    ]]]
  report growth
end

; add new houses on the market in each period if household wishes to move or if debt payments too high
to newListings
  ask households [
    let ageInt (age - age mod 10 - 20) / 10
    let probMove item ageInt movingProbability
    let rand random-float 1
    let preferred preferredExp income
    if ((rand < probMove or income - (interestRate * debt) < item (year - 1) consumptionBudget / 2) and myHome != nobody) [
      ask myHome [
        set onMarket 1
        set listingPrice getListing self
      ]
     ]
  ]
end

; calculate listing price of dwelling to be put on the market
to-report getListing [dwelling]
  let averageSqm [listingPrice] of dwelling / [sqm] of dwelling
  ifelse one-of quarterSales with [zone = [zone] of dwelling and houseCategory = [houseCategory] of dwelling] != nobody [
    set averageSqm mean [salesPrice / sqm] of quarterSales with [zone = [zone] of dwelling and houseCategory = [houseCategory] of dwelling]
  ][ ifelse one-of quarterSales with [zone = [zone] of myself] != nobody [
    set averageSqm mean [salesPrice / sqm] of quarterSales with [zone = [zone] of dwelling]
  ][ ifelse count quarterSales > 0 [
    set averageSqm mean [salesPrice / sqm] of quarterSales
  ][ set averageSqm listingPrice / sqm
  ]]]
  report averageSqm * sqm
end

; let households find potential houses to bid on
to placeBids
  ask households with [myHome = nobody and approvedMortgage != nobody and approvedMortgage != 0] [
    let maxMortgage [principle] of approvedMortgage
    let myLTV item (year - 1) ltvLimit
    let potentialHouses houses with [(onMarket = 1) and ([savings] of myself + maxMortgage > listingPrice) and ([savings] of myself / listingPrice > 1 - myLTV)]
    set bidList sort-by [([listingPrice] of ?1 > [listingPrice] of ?2)] potentialHouses
    if length bidList > nSearch [
      set bidList sublist bidlist 0 nSearch
    ]
    if age <= 60 and hasChild = 1 [
      set bidList sort-by [([houseCategory] of ?1 < [houseCategory] of ?2) or ([houseCategory] of ?1 = [houseCategory] of ?2 and [distCenter] of ?1 < [distCenter] of ?2)] potentialHouses
    ]
    if age <= 60 and hasChild = 0 [
      set bidList sort-by [([distCenter] of ?1 < [distCenter] of ?2)  or ([distCenter] of ?1 = [distCenter] of ?2 and [houseCategory] of ?1 < [houseCategory] of ?2)] potentialHouses
    ]
    if age > 60 [
      set bidList sort-by [([houseCategory] of ?1 > [houseCategory] of ?2)  or ([houseCategory] of ?1 = [houseCategory] of ?2 and [distCenter] of ?1 < [distCenter] of ?2)] potentialHouses
      ]
  ]
  ask households with [myHome != nobody and approvedMortgage != nobody and approvedMortgage != 0] [
    let maxMortgage [principle] of approvedMortgage
    let investmentHouses houses with [ (myOwner = nobody) and ((yearRent + (yearAppreciation - 1) * listingPrice - 0.10 * listingPrice  * interestRate * (1 - 0.28)) / (0.10 * listingPrice) > minYield) and ([savings] of myself + maxMortgage > listingPrice) and ([savings] of myself / listingPrice > 1 - item (year - 1) ltvLimit)]
    if investmentHouses != nobody [
      set bidList sort-by [([listingPrice] of ?1 > [listingPrice] of ?2) ] investmentHouses
    ]
  ]
end

; let households place bids
to bidRound
  ask households with [length bidList > 0] [
    let index 0
    while [(index < length bidList) and ([onMarket] of item index bidList = 0)] [
      set index index + 1
    ]
    if index > 0 and index < length bidList [
       set bidList sublist bidlist index (length bidList)
    ]
    if index = length bidList [
      set bidList []
    ]
    if length bidList > 0 [
      let price [listingPrice] of item 0 bidList
      set currentBid price * 0.95  + random-float 1 * (price * 0.1)
      ask item 0 bidList [
        set bidders lput myself bidders
      ]
    ]
  ]
end

; clear market by multiple bidding rounds
to clearMarket
  placeBids
  bidRound
  while [count houses with [length bidders > 0] > 0] [
    ask houses with [length bidders > 0] [
      let sortedList sort-by [[currentBid] of ?1 > [currentBid] of ?2] bidders
      ifelse length sortedList > 1 [
        set salesPrice [currentBid] of item 1 sortedList
        ][
        set salesPrice listingPrice * 0.95
        ]
      if myOwner != nobody [
        updateSeller myOwner self
      ]
      setColor salesPrice
      set myOwner item 0 sortedList
      set onMarket 0
      set timeOnMarket 0
      set soldAt ticks
      set bidders []
      updateBuyer myOwner self
      if [myHome] of myOwner != self [
        set isRental 1
      ]
    ]
  ask households with [length bidList > 0][
    set bidList remove-item 0 bidList
    set currentBid 0
  ]
  bidRound
  ]
  updateSales
end


; update salesvectors used for plotting
to updateSales
  set periodSales houses with [soldAt = ticks]
  set quarterSales houses with [soldAt = ticks or soldAt = ticks - 1 or soldAt = ticks - 2 or soldAt = ticks - 3]
  set yearSales houses with [soldAt > ticks - 12]
end


; help function for bidding round to update seller of dwelling
to updateSeller [owner sale]
 ask owner [
   ifelse myHome = sale [
     set myHome nobody
   ][
     set myInvestments remove sale myInvestments
   ]
   let currentMortgage one-of myMortgages with [mortgageHouse = sale]
   ifelse currentMortgage != nobody [
       set savings savings + [salesPrice] of sale - [principle] of currentMortgage
       set debt debt - [principle] of currentMortgage
       ask currentMortgage [
         die
       ]
     ][
       set savings savings + [salesPrice] of sale
     ]
 ]
end

; help function for bidding round to update buyer of dwelling
to updateBuyer [buyer sale]
  ask buyer [
    ifelse myHome = nobody [
      set myHome sale
    ][
      set myInvestments lput sale myInvestments
    ]
    if myRental != nobody [
      ask myRental [
        set myRenter nobody
      ]
      set myRental nobody
    ]
    set bidList []
    set currentBid 0
    ifelse savings > [salesPrice] of sale [
      set savings savings - [salesPrice] of sale
    ][
      let newDebt ([salesPrice] of sale) - savings
      set debt debt + newDebt
      set savings 0
      setupMortgage approvedMortgage newDebt ([salesPrice] of sale) sale
    ]
  ]
end

; setup mortgage for buyer of dwelling
to setupMortgage [approved value price sold]
  ask approved [
    set mortgageHouse sold
    set principle value
    set LTV (value / price)
  ]
  ask [mortgageOwner] of approved [
    set myMortgages mortgages with [mortgageOwner = myself]
  ]
end

; calculate appreciation on houses used for investment decisions
to calcAppreciation
  if ticks > 16 [
    set yearAppreciation mean [salesPrice / sqm] of houses with [soldAt < ticks and soldAt > ticks - 6] / mean [salesPrice / sqm] of houses with [soldAt < ticks - 12 and soldAt > ticks - 16 ]
  ]
end

; let household rent dwelling if they cannot afford to buy
to rentHome
  ask households with [myHome = nobody] [
    if one-of houses with [isRental = 1 and myRenter = nobody] != nobody [
      set myRental one-of houses with [isRental = 1 and myRenter = nobody] != nobody
      ask myRental [
        set myRenter myself
      ]
    ]
  ]
end


; update rental cost for rentals on the market
to updateRents
  ask houses [
    let averageSqm listingPrice / sqm
    ifelse one-of quarterSales with [zone = [zone] of myself and houseCategory = [houseCategory] of myself] != nobody [
          set averageSqm mean [salesPrice / sqm] of quarterSales with [zone = [zone] of myself and houseCategory = [houseCategory] of myself]
        ][ ifelse one-of quarterSales with [zone = [zone] of myself] != nobody [
          set averageSqm mean [salesPrice / sqm] of quarterSales with [zone = [zone] of myself]
        ][ ifelse count quarterSales > 0 [
          set averageSqm mean [salesPrice / sqm] of quarterSales
        ][ set averageSqm listingPrice / sqm
        ]]]
    set yearRent averageSqm * sqm * (interestRate)
  ]
end


; update listing price of houses on the market
to updateListings
  ask houses with [timeOnMarket > 3] [
    if myOwner != nobody[
        set listingPrice listingPrice * 0.99
      ]
  ]
  ask houses with [onMarket = 1][
    set timeOnMarket timeOnMarket + 1
  ]
end

; get age-dependent savings rate
to-report getSavings [ages]
  let rate 0
  ifelse ages < 24 [
      set rate item 0 savingsRates
    ][ ifelse ages < 34 [
      set rate item 1 savingsRates
    ][ ifelse ages < 44 [
      set rate item 2 savingsRates
    ][ ifelse ages < 54 [
      set rate item 3 savingsRates
    ][ ifelse ages < 64 [
      set rate item 4 savingsRates
    ][ ifelse ages < 74 [
      set rate item 5 savingsRates
    ][ set rate item 6 savingsRates ]
    ]]]]]
    report rate
end

; update savings of households
to updateSavings
  ask households with [count myMortgages = 0] [
    set savings savings + (1 / 12) * (getSavings age) * income
  ]
end

; let households pay debt and save remains
to payDebt
  ask households with [count myMortgages > 0] [
    ask myMortgages [
      let periodPayment (principle * monthInterest) / (1 - (1 + monthInterest)^(- maturity))
      set downPayment periodPayment - (principle * monthInterest)
      set principle principle - downPayment
      set maturity maturity - 1
    ]
    let monthSavings (1 / 12) * (getSavings age) * income
    if sum [downPayment] of myMortgages < monthSavings [
      set savings savings + monthSavings - sum [downPayment] of myMortgages
    ]
    set debt debt - sum [downPayment] of myMortgages
  ]
end


; add houses to model based on historical figures
to buildHouses ; Distribute total number of houses constructed in each year over the months of the simulation
  let indexStart item (year - 1) yearBuilds
  let indexStop item year yearBuilds
  let periodBuilds round((indexStop - indexStart) / 12)
  let yearBuilt periodBuilds * (month - 1)
  if ticks mod 12 = 0 and (indexStop - indexStart) > 0[
    set periodBuilds (indexStop - indexStart) - 11 * periodBuilds  ; If last period in year, build remaining houses
  ]
  let built 0
  while [built < periodBuilds] [
    let buildParams item (indexStart + yearBuilt + built) construction ; Get parameters to build one house from list of constructions
    create-houses 1 [
      set houseCategory item 0 buildParams
      set zone item 2 buildParams
      set sqm item 3 buildParams
      set listingPrice item 1 buildParams
      set listingPrice getListing self
      set salesPrice listingPrice
      set timeOnMarket 0
      set onMarket 1
      set bidders []
      set myOwner nobody
      set timeOnMarket 0
      move-to one-of patches with [inZone = [zone] of myself]
      set distCenter distance center
    ]
    set built built + 1
  ]

end


; add immigration to model matching data - only young households as net immigration for older households negilable
to updateImmigration
  let monthImmigrants round((item (year - 1) netImmigration) / 12)
  create-households monthImmigrants [
    set age 20 + random 10
    set householdCategory 1
    set income one-of [income] of households with [age <= 30]
    set savings one-of [savings] of households with [age <= 30]
    set incomePrev income
    set myHome nobody
    set bidList []
    set myInvestments []
    set myMortgages no-turtles
    set myRental nobody
    ]
end












;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Plotting
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



to setup-housing-price-plot
  set-current-plot "Housing price index"
  set-plot-y-range  0 10
end

to update-housing-price-plot
  set-current-plot "Housing price index"
  let index 0
  if count periodSales > 0 [
    set index mean [salesPrice / sqm] of periodSales
  ]
  set-current-plot-pen "index"
  plot index
end

to update-year-price-plot
  set-current-plot "Yearly index"
  let yearI mean [salesPrice / sqm] of yearSales
  set-current-plot-pen "yearI"
  plot yearI
end

to total-debt-plot
 set-current-plot "Total debt"
  let totalDebt mean [debt] of households
  set-current-plot-pen "totalDebt"
  plot totalDebt
end

to update-mean-DTI-plot
  set-current-plot "Mean DTI"
  let meanDTI mean [initDTI] of mortgages with [timeStamp < ticks and timeStamp > ticks - 13 and ageHousehold < 35 and mortgageHouse != nobody]
  set-current-plot-pen "meanDTI"
  plot meanDTI
end



to update-daysOnMarket-plot
  set-current-plot "Average time on market"
  let timeSales 0;
  if houses with [soldAt = ticks - 1] != nobody [
    set timeSales mean [timeOnMarket] of houses with [soldAt = ticks - 1] ]
  set-current-plot-pen "time"
  plot timeSales
end

to update-ownership-plot
  set-current-plot "Home ownership rate"
  let owners count households with [myHome != nobody] / count households
  set-current-plot-pen "owners"
  plot owners
end

to update-rentals-plot
  set-current-plot "Household savings"
  let householdSavings mean [savings] of households
  set-current-plot-pen "savings"
  plot householdSavings
end


to update-salesListing-plot
  set-current-plot "Sales/listingprice"
  if houses with [soldAt = ticks - 1] != nobody [
  let salesListing mean [salesPrice / listingPrice] of houses with [soldAt = ticks - 1]
  set-current-plot-pen "salesListing"
  plot salesListing
  ]

end
@#$#@#$#@
GRAPHICS-WINDOW
730
69
1228
588
50
50
4.832
1
10
1
1
1
0
0
0
1
-50
50
-50
50
1
1
1
ticks
30.0

BUTTON
23
71
86
104
NIL
setup\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
104
71
190
104
simulation
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

TEXTBOX
344
48
494
66
NIL
11
0.0
1

PLOT
221
70
458
218
Housing price index
Month
Index
0.0
100.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""
"index" 1.0 0 -7500403 true "" ""

PLOT
475
230
714
388
Total debt
NIL
NIL
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""
"totalDebt" 1.0 0 -16777216 true "" ""

TEXTBOX
27
41
177
59
Run model
13
0.0
1

PLOT
221
229
459
388
Household savings
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""
"savings" 1.0 0 -16777216 true "" ""

PLOT
475
400
715
556
Home ownership rate
NIL
NIL
0.0
100.0
0.0
1.0
true
false
"" ""
PENS
"owners" 1.0 0 -16777216 true "" ""

PLOT
223
400
457
555
Mean DTI
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"meanDTI" 1.0 0 -16777216 true "" ""

PLOT
21
193
198
343
Income Distribution
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"set-plot-x-range 50000 850000\nset-plot-y-range 0 500\nset-histogram-num-bars 8" ""
PENS
"default" 1.0 0 -16777216 true "" "histogram [income] of households"

PLOT
21
348
198
498
Age Distribution
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"set-plot-x-range 0 80\nset-plot-y-range 0 1000\nset-histogram-num-bars 100" ""
PENS
"default" 1.0 0 -16777216 true "" "histogram [age] of households"

SLIDER
22
125
194
158
DTI
DTI
0
10
5
1
1
NIL
HORIZONTAL

PLOT
477
574
714
724
HousePrices
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"set-plot-x-range 0 10000000\nset-plot-y-range 0 500\nset-histogram-num-bars 50" ""
PENS
"default" 1.0 0 -16777216 true "" "histogram [listingPrice] of houses"

PLOT
223
573
457
723
Sales/Listingprice
NIL
NIL
0.0
10.0
0.0
1.2
true
false
"" ""
PENS
"salesListing" 1.0 0 -16777216 true "" ""

PLOT
474
70
714
218
Yearly Index
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"yearI" 1.0 0 -16777216 true "" ""

@#$#@#$#@
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="50" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>periodSqm</metric>
    <enumeratedValueSet variable="DTI">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="maxLTV">
      <value value="1"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
